#-*- coding:utf8 -*-

'''
Hessian2.0 decoder implementation in python.

According to http://hessian.caucho.com/doc/hessian-serialization.html.
'''

from struct import unpack
from datetime import datetime
MKTIME = datetime.utcfromtimestamp
from .proto import HessianObjectFactory, TypedMap


ONE_INT_CODE_RANGE = ('\x80', '\xbf')
TWO_INT_CODE_RANGE = ('\xc0', '\xcf')
THREE_INT_CODE_RANGE = ('\xd0', '\xd7')

ONE_LONG_CODE_RANGE = ('\xd8', '\xef')
TWO_LONG_CODE_RANGE = ('\xf0', '\xff')
THREE_LONG_CODE_RANGE = ('\x38', '\x3f')
# ONE TWO THREE is short for ONE_OCTET TWO_OCTET THREE_OCTET

SHORT_STRING_CODE_RANGE = ('\x00', '\x1f')
SHORT_BINARY_CODE_RANGE = ('\x00', '\x1f')


class Decoder(object):
    def __init__(self):
        self.hessian_obj_factory = HessianObjectFactory()
        self._refs = []
        self.decoders = {
            'N': self.decode_null,
            'T': self.decode_bool,
            'F': self.decode_bool,
            'w': self.decode_int,
            'Y': self.decode_long,
            'L': self.decode_long,
            'D': self.decode_double,
            '\x5b': self.decode_double,
            '\x5c': self.decode_double,
            '\x5d': self.decode_double,
            '\x5e': self.decode_double,
            '\x5f': self.decode_double,
            #'B': self.decode_binary,
            #'b': self.decode_binary,
            '\x4b': self.decode_ref,
            '\x4a': self.decode_ref,
            'd': self.decode_date,  # compatible with hessian 1.0
            'V': self.decode_list,
            'S': self.decode_string,
            'M': self.decode_typed_map,
            'H': self.decode_untyped_map,
            'O': self.decode_object,
            'o': self.decode_object_instance,
            '\x51': self.decode_ref,
        }

    def _set_decoder(self, byte_code, decoder_name):
        if not hasattr(self, decoder_name):
            raise Exception('Unknown decoder name: %s' % decoder_name)
        self.decoders[byte_code] = getattr(self, decoder_name)

    def decode(self, buf):
        return self._decode(0, buf)[1]

    def _decode(self, pos, buf):
        tag = buf[pos]
        if tag in self.decoders:
            return self.decoders[tag](pos, buf)
        elif self.is_int(tag):
            return self.decode_int(pos, buf)
        elif self.is_long(tag):
            return self.decode_long(pos, buf)
        elif self.is_string(tag):
            return self.decode_string(pos, buf)
        elif self.is_binary(tag):
            return self.decode_binary(pos, buf)
        else:
            # TODO: decode following code range
            '''
            x30 - x33    # utf-8 string length 0-1023
            x34 - x37    # binary data length 0-1023
            x60 - x6f    # object with direct type
            x70 - x77    # fixed list with direct length
            x78 - x7f    # fixed untyped list with direct length
            '''
            raise Exception("decode error, unknown tag: %r" % tag)

    @staticmethod
    def is_int(tag):
        return (
            (ONE_INT_CODE_RANGE[0] <= tag <= ONE_INT_CODE_RANGE[1]) or
            (TWO_INT_CODE_RANGE[0] <= tag <= TWO_INT_CODE_RANGE[1]) or
            (THREE_INT_CODE_RANGE[0] <= tag <= THREE_INT_CODE_RANGE[1]) or
            tag == 'I'
        )

    @staticmethod
    def is_long(tag):
        return (
            (ONE_LONG_CODE_RANGE[0] <= tag <= ONE_LONG_CODE_RANGE[1]) or
            (TWO_LONG_CODE_RANGE[0] <= tag <= TWO_LONG_CODE_RANGE[1]) or
            (THREE_LONG_CODE_RANGE[0] <= tag <= THREE_LONG_CODE_RANGE[1]) or
            tag == 'Y' or tag == 'L'
        )

    @staticmethod
    def is_string(tag):
        return (
            (SHORT_STRING_CODE_RANGE[0] <= tag <= SHORT_STRING_CODE_RANGE[1])
            or tag == 's' or tag == 'S'
        )

    @staticmethod
    def is_binary(tag):
        return (
            (SHORT_BINARY_CODE_RANGE[0] <= tag <= SHORT_BINARY_CODE_RANGE[1])
            or tag == 's' or tag == 'S'
        )

    def decode_null(self, pos, buf):
        return pos+1, None

    def decode_bool(self, pos, buf):
        return pos+1, buf[pos] == 'T'

    def decode_int(self, pos, buf):
        tag = buf[pos]
        if ONE_INT_CODE_RANGE[0] <= tag <= ONE_INT_CODE_RANGE[1]:
            return pos+1, ord(tag) - 0x90
        elif TWO_INT_CODE_RANGE[0] <= tag <= TWO_INT_CODE_RANGE[1]:
            return pos+2, ((ord(tag) - 0xc8) << 8) + ord(buf[pos+1])
        elif THREE_INT_CODE_RANGE[0] <= tag <= THREE_INT_CODE_RANGE[1]:
            return pos+3, ((ord(tag)-0xd4)<<16) + (ord(buf[pos+1])<<8) + ord(buf[pos+2])
        elif tag == 'w':
            return pos+5, unpack('>l', buf[pos+1:pos+5])[0]
        else:
            raise Exception("decode int error, unknown tag: %r" % tag)

    def decode_long(self, pos, buf):
        tag = buf[pos]
        if ONE_LONG_CODE_RANGE[0] <= tag <= ONE_LONG_CODE_RANGE[1]:
            return pos+1, ord(tag) - 0xe0
        elif TWO_LONG_CODE_RANGE[0] <= tag <= TWO_LONG_CODE_RANGE[1]:
            return pos+2, ((ord(tag)-0xf8)<<8) + ord(buf[pos+1])
        elif THREE_LONG_CODE_RANGE[0] <= tag <= THREE_LONG_CODE_RANGE[1]:
            return pos+3, ((ord(tag)-0x3c)<<16) + (ord(buf[pos+1])<<8) + ord(buf[pos+2])
        elif tag == 'Y':
            return pos+5, unpack('>l', buf[pos+1:pos+5])[0]
        elif tag == 'L':
            return pos+9, unpack('>q', buf[pos+1:pos+9])[0]
        else:
            raise Exception("decode long error, unknown tag: %r" % tag)

    def decode_double(self, pos, buf):
        tag = buf[pos]
        if tag == '\x5b':
            return pos+1, 0.0
        elif tag == '\x5c':
            return pos+1, 1.0
        elif tag == '\x5d':
            return pos+2, float(unpack('>b', buf[pos+1])[0])
        elif tag == '\x5e':
            return pos+3, float(unpack('>h', buf[pos+1:pos+3])[0])
        elif tag == '\x5f':
            return pos+5, unpack('>f', buf[pos+1:pos+5])[0]
        elif tag == 'D':
            return pos+9, unpack('>d', buf[pos+1:pos+9])[0]
        else:
            raise Exception("decode double error, unknown tag: %r" % tag)

    def decode_binary(self, pos, buf):
        tag = buf[pos]
        # TODO: uncompleted
        pass

    def decode_date(self, pos, buf):
        tag = buf[pos]; pos += 1
        if tag == '\x4b':
            return pos+4, MKTIME(unpack('>l', buf[pos:pos+4])[0]*60)
        elif tag == '\x4a' or tag == 'd':
            return pos+8, MKTIME(unpack('>q', buf[pos:pos+8])[0]/1000)
        else:
            raise Exception("decode date error, unknown tag: %r" % tag)

    def decode_list(self, pos, buf):
        tag = buf[pos]; pos += 1
        if tag == 'V':
            tag = buf[pos]; pos += 1
            if tag == 't':
                type_length = (ord(buf[pos]) << 8) + ord(buf[pos+1]); pos += 2
                _type = buf[pos:(pos+type_length)]; pos += type_length
                tag = buf[pos]; pos += 1

            if tag == 'n':
                length = unpack('>B', buf[pos])[0]; pos += 1
            elif tag == 'l':
                length = unpack('>H', buf[pos: pos+2])[0]; pos += 2
            else:
                raise Exception(
                    "decode list length error, unknown tag: %r" % tag)

            ret = []
            for i in xrange(length):
                pos, obj = self._decode(pos, buf)
                ret.append(obj)
            assert buf[pos] == 'z'; pos += 1
            return pos, ret
        else:
            raise Exception("decode list error, unknown tag: %r" % tag)

    def read_characters(self, pos, buf, length):
        '''
        read length characters from buf.
        since hessian string '\x02\xe4\xb8\xad\xe6\x96\x87' represents
        2 utf8 characters, we decode '\x02\xe4\xb8' as a string.

        UTF8 characters length:
            00-7F one octet, ascii
            C0-DF two octets
            E0-EF three octets
            F0-F7 four octets
        '''
        begin = pos
        for _ in xrange(length):
            code = buf[pos]
            if code <= '\x7f':
                pos += 1
            elif '\xc0' <= code <= '\xdf':
                pos += 2
            elif '\xe0' <= code <= '\xef':
                pos += 3
            elif '\xf0' <= code <= '\xf7':
                pos += 4
            else:
                raise Exception('Unknown utf8 character: %r' % code)
        return pos, buf[begin:pos]

    def decode_string(self, pos, buf):
        tag = buf[pos]
        pos+= 1
        if SHORT_STRING_CODE_RANGE[0] <= tag <= SHORT_STRING_CODE_RANGE[1]:
            length = ord(tag)
            return self.read_characters(pos, buf, length)
        elif tag == 'S':
            length = unpack('>H', buf[pos:pos+2])[0]
            return self.read_characters(pos+2, buf, length)
        elif tag == 's':  # tag == 's'
            length = unpack('>H', buf[pos:pos+2])[0]
            pos, data = self.read_characters(pos+2, buf, length)
            pos, subdata = self.decode_string(pos, buf)
            return pos, data+subdata
        else:
            raise Exception("decode string error, unknown tag: %r" % tag)

    def decode_untyped_map(self, pos, buf):
        tag = buf[pos]; pos += 1
        ret = {}
        if tag == 'H' or tag == 'M':
            while buf[pos] != 'z':
                pos, key = self._decode(pos, buf)
                pos, value = self._decode(pos, buf)
                ret[key] = value
            return pos+1, ret
        else:
            raise Exception("decode untyped map error, unknown tag: %r" % tag)

    def decode_typed_map(self, pos, buf):
        return self.decode_untyped_map(pos, buf)
        tag = buf[pos]; pos += 1
        ret = {}
        if tag == 'M':
            pos, _type = self.decode_string(pos, buf)
            ret['type'] = _type
            ret['body'] = {}
            while buf[pos] != 'z':
                pos, key = self._decode(pos, buf)
                pos, value = self._decode(pos, buf)
                ret['body'][key] = value
            return pos+1, ret
        else:
            raise Exception("decode map error, unknown tag: %r" % tag)

    def decode_object(self, pos, buf):
        tag = buf[pos]; pos += 1
        if tag == 'O':
            pos, _class = self.decode_string(pos, buf)
            pos, field_num = self.decode_int(pos, buf)
            fields = []
            for i in xrange(field_num):
                pos, field = self.decode_string(pos, buf)
                fields.append(field)
            self.hessian_obj_factory.create_object(_class, fields)

            pos, obj = self.decode_object_instance(pos, buf)
            return pos, obj
        else:
            raise Exception("decode map error, unknown tag: %r" % tag)

    def decode_object_instance(self, pos, buf):
        tag = buf[pos]; pos += 1
        ret = []
        if tag == 'o':
            self._refs.append(None)  # occupy the position
            ref_id = len(self._refs) - 1   # record the position
            # decode ref id
            tag = buf[pos]
            ref = 0
            if self.is_int(tag):
                pos, ref = self.decode_int(pos, buf)

            values = []
            field_num = self.hessian_obj_factory.object_field_num(ref)
            for i in xrange(field_num):
                pos, value = self._decode(pos, buf)
                values.append(value)
            obj = self.hessian_obj_factory.create_instance(ref, values)
            self._refs[ref_id] = obj
            return pos, obj
        else:
            raise Exception("decode map error, unknown tag: %r" % tag)

    def decode_ref(self, pos, buf):
        tag = buf[pos]; pos += 1
        if tag == '\x51':
            pos, ref = self.decode_int(pos, buf)
        elif tag == '\x4a':
            ref = ord(buf[pos]); pos += 1
        elif tag == '\x4b':
            ref = (ord(buf[pos]) << 8) + ord(buf[pos+1]); pos += 2
        else:
            raise Exception("decode ref error, unknown tag: %r" % tag)
        return pos, self._refs[ref]



#-*- coding:utf8 -*-

'''
Hessian2.0 encoder implementation in python.

According to http://hessian.caucho.com/doc/hessian-serialization.html.

Hessian Bytecode map:
    x00 - x1f    # utf-8 string length 0-32
    x20 - x2f    # binary data length 0-16
    x30 - x33    # utf-8 string length 0-1023
    x34 - x37    # binary data length 0-1023
    x38 - x3f    # three-octet compact long (-x40000 to x3ffff)
    x40          # reserved (expansion/escape)
    x41          # 8-bit binary data non-final chunk ('A')
    x42          # 8-bit binary data final chunk ('B')
    x43          # object type definition ('C')
    x44          # 64-bit IEEE encoded double ('D')
    x45          # reserved
    x46          # boolean false ('F')
    x47          # reserved
    x48          # untyped map ('H')
    x49          # 32-bit signed integer ('I')
    x4a          # 64-bit UTC millisecond date
    x4b          # 32-bit UTC minute date
    x4c          # 64-bit signed long integer ('L')
    x4d          # map with type ('M')
    x4e          # null ('N')
    x4f          # object instance ('O')
    x50          # reserved
    x51          # reference to map/list/object - integer ('Q')
    x52          # utf-8 string non-final chunk ('R')
    x53          # utf-8 string final chunk ('S')
    x54          # boolean true ('T')
    x55          # variable-length list/vector ('U')
    x56          # fixed-length list/vector ('V')
    x57          # variable-length untyped list/vector ('W')
    x58          # fixed-length untyped list/vector ('X')
    x59          # long encoded as 32-bit int ('Y')
    x5a          # list/map terminator ('Z')
    x5b          # double 0.0
    x5c          # double 1.0
    x5d          # double represented as byte (-128.0 to 127.0)
    x5e          # double represented as short (-32768.0 to 32767.0)
    x5f          # double represented as float
    x60 - x6f    # object with direct type
    x70 - x77    # fixed list with direct length
    x78 - x7f    # fixed untyped list with direct length
    x80 - xbf    # one-octet compact int (-x10 to x3f, x90 is 0)
    xc0 - xcf    # two-octet compact int (-x800 to x7ff)
    xd0 - xd7    # three-octet compact int (-x40000 to x3ffff)
    xd8 - xef    # one-octet compact long (-x8 to xf, xe0 is 0)
    xf0 - xff    # two-octet compact long (-x800 to x7ff, xf8 is 0)
'''

from struct import pack
import datetime
import types
from .proto import HessianObject, TypedMap


ONE_OCTET_INT_RANGE = (-0x10, 0x3f)
TWO_OCTET_INT_RANGE = (-0x800, 0x7ff)
THREE_OCTET_INT_RANGE = (-0x40000, 0x3ffff)

ONE_OCTET_LONG_RANGE = (-0x8, 0xf)
TWO_OCTET_LONG_RANGE = (-0x800, 0x7ff)
THREE_OCTET_LONG_RANGE = (-0x40000, 0x3ffff)
FOUR_OCTET_LONG_RANGE = (-0x80000000, 0x7fffffff)


class Encoder(object):
    def __init__(self):
        self._refs = []
        self._classes = []
        self.encoders = {
            types.NoneType: self.encode_null,
            types.BooleanType: self.encode_bool,
            datetime.datetime: self.encode_date,
            types.IntType: self.encode_int,
            types.LongType: self.encode_long,
            types.FloatType: self.encode_double,
            types.ListType: self.encode_list,
            types.TupleType: self.encode_list,
            types.StringType: self.encode_string,
            types.UnicodeType: self.encode_string,
            types.DictType: self.encode_untyped_map,
            TypedMap: self.encode_typed_map,
            HessianObject: self.encode_object,
        }

    def encode(self, val):
        _type = type(val)
        if _type not in self.encoders:
            raise Exception("No encoder for type: %s" % _type)
        return self.encoders[_type](val)

    def encode_ref(self, val):
        # TODO: reference mark is 'Q' or 'R'? Use 'Q'
        '''
        x51          # reference to map/list/object - integer ('Q')
        '''
        _id = id(val)
        if _id in self._refs:
            return 'Q' + self.encode_int(self._refs.index(_id))
        self._refs.append(_id)

    def encode_null(self, val):
        return 'N'

    def encode_bool(self, val):
        if val:
            return 'T'
        else:
            return 'F'

    def encode_int(self, val):
        if ONE_OCTET_INT_RANGE[0] <= val <= ONE_OCTET_INT_RANGE[1]:
            # value = code - 0x90
            # b0 = b0 + 0x90
            return chr(val + 0x90)
        elif TWO_OCTET_INT_RANGE[0] <= val <= TWO_OCTET_INT_RANGE[1]:
            # value = ((code - 0xc8) << 8) + b0
            # b1 = b1 + 0xc8, b0 = b0
            return chr((val>>8) + 0xc8) + chr(val&0xff)
        elif THREE_OCTET_INT_RANGE[0] <= val <= THREE_OCTET_INT_RANGE[1]:
            # value = ((code - 0xd4) << 16) + (b1 << 8) + b0;
            # b2 = b2 + 0xd4, b1 = b1, b0 = b0
            return chr((val>>16) + 0xd4) + chr((val>>8)&0xff) + chr(val&0xff)
        else:
            return pack('>cl', 'w', val)

    def encode_long(self, val):
        '''
        L b7 b6 b5 b4 b3 b2 b1 b0
        xd8 - xef    # one-octet compact long (-x8 to xf, xe0 is 0)
        xf0 - xff    # two-octet compact long (-x800 to x7ff, xf8 is 0)
        x38 - x3f    # three-octet compact long (-x40000 to x3ffff)
        x59          # long encoded as 32-bit int ('Y')
        x4c          # 64-bit signed long integer ('L')
        '''
        if ONE_OCTET_LONG_RANGE[0] <= val <= ONE_OCTET_LONG_RANGE[1]:
            # value = (code - 0xe0)
            # b0 = b0 + 0xe0
            return chr(val + 0xe0)
        elif TWO_OCTET_LONG_RANGE[0] <= val <= TWO_OCTET_LONG_RANGE[1]:
            # value = ((code - 0xf8) << 8) + b0
            # b1 = b1 + 0xf8, b0 = b0
            return chr((val>>8) + 0xf8) + chr(val&0xff)
        elif THREE_OCTET_LONG_RANGE[0] <= val <= THREE_OCTET_LONG_RANGE[1]:
            # value = ((code - 0x3c) << 16) + (b1 << 8) + b0
            # b2 = b2 + 0x3c, b1 = b1, b0 = b0
            return chr((val>>16) + 0x3c) + chr((val>>8)&0xff) + chr(val&0xff)
        elif FOUR_OCTET_LONG_RANGE[0] <= val <= FOUR_OCTET_LONG_RANGE[1]:
            # value = (b3 << 24) + (b2 << 16) + (b1 << 8) + b0
            return pack('>cl', '\x59', val)
        else:
            return pack('>cq', 'L', val)

    def encode_double(self, val):
        '''
        x44          # 64-bit IEEE encoded double ('D')
        x5b          # double 0.0
        x5c          # double 1.0
        x5d          # double represented as byte (-128.0 to 127.0)
        x5e          # double represented as short (-32768.0 to 32767.0)
        x5f          # double represented as float
        '''
        if val == 0.0:
            return '\x5b'
        elif val == 1.0:
            return '\x5c'

        if val.is_integer():
            _v = int(self, val)
            if -128 <= _v <= 127:
                return pack('>cb', '\x5d', _v)
            elif -32768 <= _v <= 32767:
                return pack('>ch', '\x5e', _v)

        try:
            return pack('>cf', '\x5f', val)
        except OverflowError:
            return pack('>cd', 'D', val)

    def encode_date(self, val):
        return pack('>cq', 'd', int(time.mktime(val.timetuple())) * 1000)

    def encode_binary(self, val):
        # TODO: non-final chunk mark is 'A' or 'b'? Use 'b'
        '''
        x20 - x2f    # binary data length 0-16
        x34 - x37    # binary data length 0-1023
        x41          # 8-bit binary data non-final chunk ('A')
        x42          # 8-bit binary data final chunk ('B')
        '''
        length = len(val)
        if length <= 15:
            return chr(length + 0x20) + val

        #if length <= 1023:
        #    # TODO: implement this by guess, find no spec to follow
        #    # length 0x3400 = 0, 0x37ff = 1023
        #    return chr((length>>8)+0x34) + chr(length&0xff) + val

        data = []
        index = 0
        chunk_max_size = 0xffff
        while length > chunk_max_size:
            data.append(pack('>cH','b',chunk_max_size))
            data.append(val[index:chunk_max_size])
            index += chunk_max_size
            length -= chunk_max_size

        # length must be in [1, chunk_max_size]
        data.append(pack('>cH', 'B', length))
        data.append(val[index:])
        return "".join(data)

    def encode_string(self, val):
        # TODO: non-final chunk mark is 'R' or 's'? Use 's'
        '''
        x00 - x1f    # utf-8 string length 0-32
        x30 - x33    # utf-8 string length 0-1023
        x52          # utf-8 string non-final chunk ('R')
        x53          # utf-8 string final chunk ('S')
        '''
        if isinstance(val, unicode):
            length = len(val)
            val = val.encode('utf8')
        elif isinstance(val, str):
            length = len(val.decode('utf8'))
        else:
            raise Exception(
                'encode string error, unknown type: %s' % type(val))
        if length <= 31:
            return chr(length) + val

        #if length <= 1023:
        #    # TODO: implement this by guess, find no spec to follow
        #    # length 0x3000 = 0, 0x33ff = 1023
        #    return chr((length>>8)+0x30) + chr(length&0xff) + val

        data = []
        index = 0
        chunk_max_size = 0xffff
        while length > chunk_max_size:
            data.append(pack('>cH','s',chunk_max_size))
            data.append(val[index:chunk_max_size])
            index += chunk_max_size
            length -= chunk_max_size

        # length must be in range [1, chunk_max_size]
        data.append(pack('>cH', 'S', length))
        data.append(val[index:])
        return "".join(data)

    def encode_list(self, val):
        ret = self.encode_ref(val)
        if ret:
            return ret
        length = len(val)
        data = []
        if length <= 0xff:
            data.append(pack('>2cB', 'V', 'n', length))
        else:
            data.append(pack('>2cH', 'V', 'l', length))

        for v in val:
            data.append(self.encode(v))
        data.append('z')
        return "".join(data)

    def encode_untyped_map(self, val):
        ret = self.encode_ref(val)
        if ret:
            return ret
        data = []
        data.append('H')
        for k, v in val.iteritems():
            data.append(self.encode(k))
            data.append(self.encode(v))
        data.append('z')
        return "".join(data)

    def encode_typed_map(self, val):
        ret = self.encode_ref(val)
        if ret:
            return ret
        _type, val= val._type, val.val
        data = []
        data.append('M')
        data.append(self.encode(_type))
        for k, v in val.iteritems():
            data.append(self.encode(k))
            data.append(self.encode(v))
        data.append('z')
        return "".join(data)

    def encode_object_class(self, val):
        data = []
        _class, attrs = val._class, val.attrs
        if _class in self._classes:
            return self._classes.index(_class), data

        data.append('O')
        data.append(self.encode_string(_class))
        length = len(attrs)
        data.append(self.encode_int(length))
        for k in attrs.iterkeys():
            data.append(self.encode_string(k))
        self._classes.append(_class)
        return len(self._classes) - 1, data

    def encode_object(self, val):
        ret = self.encode_ref(val)
        if ret:
            return ret
        ref_id, data = self.encode_object_class(val)
        data.append('o')
        attrs = val.attrs
        data.append(self.encode_int(ref_id))
        for v in attrs.itervalues():
            data.append(self.encode(v))
        return "".join(data)



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

VERSION = '1.0.3'

LONG_DESCRIPTION = '''
pyhessian2 is implemented for serialize and deserialize data in hessian2 protocol.

Usage
-----

    >>> # encoding
    >>> from pyhessian2 import HessianObject, Encoder
    >>> attrs = {
            "name": "xx",
            "age": 20,
        }
    >>> obj = HessianObject("com.xx.person", attrs)
    >>> data = Encoder().encode(obj)
    >>> print "%r" % data


    >>> # decoding
    >>> from pyhessian2 import Decoder
    >>> data = ...  # a hessian bytes data
    >>> obj = Decoder().decoder(data)  # get a Hessianobject instance
    >>> print obj  # print json serialized data


'''

setup(
    name='pyhessian2',
    description='an implementation for hessian2',
    long_description=LONG_DESCRIPTION,
    author='WKPlus',
    url='https://github.com/WKPlus/pyhessian2.git',
    license='MIT',
    author_email='qifa.zhao@gmail.com',
    version=VERSION,
    packages = ['pyhessian2'],
    install_requires=[],
    test_requires=['nose'],
    zip_safe=False,
)



#-*- coding:utf8 -*-

'''
Hessian2.0 encoder implementation in python.

According to http://hessian.caucho.com/doc/hessian-serialization.html.
'''

import json
from datetime import datetime


class JsonEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, datetime):
            return o.strftime('%Y-%m-%d %H:%M:%S')
        return o.__dict__


class HessianObject(object):
    def __init__(self, _class, attrs):
        self._class = _class
        self.attrs = attrs

    def representation(self):
        return {
            '_class': self._class,
            'attrs': self.attrs,
        }

    def __str__(self):
        return json.dumps(self.representation(), cls=JsonEncoder,
                          ensure_ascii=False, indent=2)


class TypedMap(object):
    def __init__(self, _type, val):
        self._type = _type
        self.val = val


class HessianObjectFactory(object):
    def __init__(self):
        self.objects = []
        self.object_fields = {}

    def create_object(self, _class, fields):
        self.objects.append(_class)
        self.object_fields[_class] = fields

    def object_field_num(self, ref):
        return len(self.object_fields[self.objects[ref]])

    def create_instance(self, ref, values):
        assert self.object_field_num(ref) == len(values)
        _class = self.objects[ref]
        val = dict(zip(self.object_fields[_class], values))
        return HessianObject(_class, val)
